%define patch_name kernel-feat-drivers-console-unicode

Name: %patch_name
Version: 1999.08.09
Release: alt12
# NOTE: libkbd_values-devel subpackage has its own version

Summary: Fixes the imperfectness of Linux VT/console Unicode support
License: GPL
Group: Development/Other
Url: ftp://ftp.ilog.fr/pub/Users/haible/utf8/
Packager: Kernel Maintainers Team <kernel@packages.altlinux.org>

BuildArch: noarch
BuildRequires: kernel-build-tools

# Applying this patch makes the default kernel-internal
# keymaps be recreated, they must use macros from kbd_values.h;
# this requirement is supported by a feature of console-tools/loadkeys:
Requires: loadkeys-mktable-macros

#source 1 linux-2.4.22-console.diff
# it has been divided into 3 parts for maintenance reasons,
# will merge them into 2 patches in %%build:
Source1: linux-2.4.22-console_common2.4or2.6.diff
Source2: linux-2.4.26-console_special2.4.diff
Source3: linux-2.6.10-console_special2.6.diff
%define patch_console_2_4 linux-2.4-console.diff
%define patch_console_2_6 linux-2.6-console.diff

# 2.4.22 keyboard-uni patch: 4 + 5
# 2.6.5 keyboard-uni patch: 4 + 6 + 7
Source4: kbd_values.h
%source 5 linux-2.4.22-keyboard-uni-private_part1.diff 02_2.4
%source 6 linux-2.6.10-keyboard-uni-private_part1.diff 02_2.6
%source 7 linux-2.6.10-keyboard-uni-private_delmaps.diff 02_2.6

# The next change is in 2.6 mainstream (checked for 2.6.5), 
# so souece8 is only for 2.4 (would be better
# to backport the 2.6 variant to 2.4 because it is cleaner):
%source 8 linux-2.4.22-tty.diff 03_2.4
# Consider this issue later (not applying for now):
Source9: linux-2.6.5-tty_more.diff

Source10: tests.txt
Source11: test-stickyshift-uni.kmap
Source12: test-latin-shift-uni.kmap
Source13: test-results-on-patched.txt
Source14: test-results-2.6.txt
Source15: bad-cases-for-kernel-kbd.txt
Source16: issues.txt

Source20: stty.diff

# Original (2.4.22 adapted) patch from Bruno Haible,
# replaced by my linux-2.4.22-keyboard-uni-private.diff
Source21: linux-2.4.22-keyboard.diff

# The change is in 2.6 even without kernel-feat-drivers-console-unicode:
Source22: man-pages-1.66-alt-unicode-tty.patch
Source23: man-pages-1.66-alt-unicode-console.patch

%package -n libkbd_values-devel
Version: 0.1.3
Group: Development/Kernel
Summary: Defines the coding of keysym values used by the kernel-user interface

%description
Linux 2.4 console and even all VTs (xterm, telnet etc.) cannot be used
smoothly in UTF-8 mode because of the imperfectness of the kernel drivers.

The issues these patches deal with:

- general VT-related:
  + correct positioning and deletion when processing UTF-8 in the "cooked"
    (usual) mode of VT line editor (is in 2.6 mainstream);

- linux console specific:
  + ESC-sequence to save/restore UTF-8/8-bit mode together with the rest
    of the VT state;
  + possibility to cut/paste using selection between ASCII and UTF console;
  + using a single keyboard map for UNICODE & 8-bit modes (without reloading);
  + CapsLock & Unicode in linux console;
  + interaction between "sticky" modifier keys and Unicode letters;
  + covering all public Unicode 2-byte characters (including previously
    unavailable Presentation and Half-/Fullwidth Forms and other).

The improvement of the VT line editor can be switched on with a
patched version of stty program (the corresponding patch is included
in docs).  Also a little addition to the documentation is suggested to
reflect the changes.

The features can be exploited by console-tools/kbd utilities (the last
three features rely upon a change of the kernel interface; so to use
it in full power loadkeys should be aware of it).

These issues are described and 2 of these
patches are recommended by the Unicode HOWTO
(see http://www.tldp.org/HOWTO/Unicode-HOWTO-2.html#ss2.1).
Authors are Edmund Thomas, Grimley Evans, Stanislav Voronyi, Bruno Haible.

%description -n libkbd_values-devel
C macro definitions corresponding to the coding of keysym values
used by the kernel-user interface of a kernel
with applied patches from %patch_name.

Should be used by such programs as loadkeys from console-tools.

Also includes macros for keysym value storage, can be useful for
console-tools as well.

%prep
%setup -c -T
%__install -m0644 %SOURCE10 %SOURCE11 %SOURCE12 %SOURCE13 %SOURCE14 \
	%SOURCE15 %SOURCE16 \
 	%SOURCE20 %SOURCE21 %SOURCE22 \
	./

%build
# merge patches -- for each kernel branch
# (we do not use the directory-per-branch mechanism
# present in kernel-build-tools for this task,
# because this is logically one single patch
# that shouldn't appear as separate pieces,
# but it has been divided for maintainance reasons):
{ \
	cat %SOURCE1; \
	cat %SOURCE2; \
} > %patch_console_2_4
{ \
	cat %SOURCE1 | sed -e 's:vt.c:vt_ioctl.c:g;s:console.c:vt.c:g'; \
	cat %SOURCE3; \
} > %patch_console_2_6

# make a patch out of a stand-alone .h-file:
%__mkdir none
%__install -pD -m644 %SOURCE4 linux/include/linux/kbd_values.h
diff -duNr none linux \
	> linux-2.4.22-keyboard-uni-private_part2.diff \
&& [[ $? == 1 ]] # treat non-standard return code of `diff'.

%install
%install_patches

%__mkdir_p %patches_dir/%patch_name/01_2.{4,6}
%__install -p -m644 %patch_console_2_4 \
	%patches_dir/%patch_name/01_2.4/
%__install -p -m644 %patch_console_2_6 \
	%patches_dir/%patch_name/01_2.6/

%__mkdir_p %patches_dir/%patch_name/02_common
%__install -p -m644 linux-2.4.22-keyboard-uni-private_part2.diff \
	%patches_dir/%patch_name/02_common/

%__mkdir_p "$RPM_BUILD_ROOT"/%_includedir/kbd_values/
%__install -p -m644 %SOURCE4 \
	"$RPM_BUILD_ROOT"/%_includedir/kbd_values/kbd_values.h

%files
%_usrsrc/*
%doc tests.txt test-stickyshift-uni.kmap test-latin-shift-uni.kmap

%files -n libkbd_values-devel
%_includedir/kbd_values

%changelog
* Sun Feb  6 2005 Ivan Zakharyaschev <imz@altlinux.ru> 1999.08.09-alt12
- Replaced linux-2.6.5 patches with linux-2.6.10 patches:
  + console_special2.6.diff:  regenerated 
    (similar to alt8 for 2.4 branch; I'm not sure whether 
    it is _kernel-fix-security_ 
    that caused the change of the tty->ldisc handling code);
  + keyboard-uni-private_delmaps.diff, keyboard-uni-private_part1.diff:
    trivial.

* Sat Feb  5 2005 Ivan Zakharyaschev <imz@altlinux.ru> 1999.08.09-alt11
- Cosmetical internal changes:
  + correct spelling;
  + kbd_values.h metadata: put CVS Id and description to the front;
  + linux-2.4.22-keyboard.diff: put a change note I forgot.

* Sat Dec 25 2004 Sergey Vlasov <vsu@altlinux.ru> 1999.08.09-alt10
- Updated linux-2.4.26-console_special2.4.diff:
  + fixed for compatibility with new kernel-fix-security

* Sun Dec 05 2004 Sergey Vlasov <vsu@altlinux.ru> 1999.08.09-alt9
- Removed explicit "Release: alt1" from the libkbd_values-devel subpackage.

* Mon Nov 29 2004 Sergey Vlasov <vsu@altlinux.ru> 1999.08.09-alt8
- Replaced linux-2.4.22-console_special2.4.diff with
  linux-2.4.26-console_special2.4.diff:
  + regenerated patch for compatibility with new kernel-fix-security

* Fri Jun 18 2004 Sergey Vlasov <vsu@altlinux.ru> 1999.08.09-alt7
- Removed fake tarballs; added kbd_values.h as a source file.
- include/linux/kbd_values.h:
  + replaced "inline" with "__inline__" to fix userspace compilation

* Thu Jun  3 2004 Ivan Zakharyaschev <imz@altlinux.ru> 1999.08.09-alt6
- include/linux/kbd_values.h:
  + rewrite K_UNI_STRIP_WITHCASE() using macros 
    (similar to K_UNI_ADD_WITHCASE());
  + by default, disable the areas_withcase[] array definition.

* Wed Jun 02 2004 Sergey Vlasov <vsu@altlinux.ru> 1999.08.09-alt5
- include/linux/kbd_values.h:
  + really fixed userspace compilation problem (need to use __u16, not uint16_t
    in kernel headers)
- cleanup_spec applied
- %%description reformatted

* Mon May 17 2004 Sergey Vlasov <vsu@altlinux.ru> 1999.08.09-alt4
- include/linux/kbd_values.h:
  + fixed userspace compilation problem (missing #include <linux/types.h>)
  + fixed "comparison between signed and unsigned" warning

* Fri Apr 23 2004 Ivan Zakharyaschev <imz@altlinux.ru> 1999.08.09-alt3
- added 2.6-branch variants;

* Fri Apr 16 2004 Ivan Zakharyaschev <imz@altlinux.ru> 1999.08.09-alt2
- extended keyboard patch (moving values in Unicode private area
  and all the related changes even in the interface);
- libkbd_values-devel: the corresponding macro definitions
  (to be used by loadkeys).

* Wed Mar 17 2004 Ivan Zakharyaschev <imz@altlinux.ru> 1999.08.09-alt1
- adapted the 2.3 patches collected by Bruno Haible to 2.4.22;
- included a detailed test description for each feature the pacthes add.
